local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Table = BaikCore.Table
local Ui = BaikCore.Ui

local FrameStyle = BaikActionbar.FrameStyle

--Add Module
local ActionBar= BaikActionbar:NewModule("ActionBar")

-- Constants
local ACTION_PREFIX = "ActionButton%d"

-- Private Variables
local actionBtns = nil

-- Private Methods
local function GetButtons()
    if actionBtns then
        return actionBtns
    end

    actionBtns = {}
    for idx=1, NUM_ACTIONBAR_BUTTONS, 1 do
        local btnName = string.format(ACTION_PREFIX, idx)
        local btn = _G[btnName]
        Assert:NotNull(btn)
        table.insert(actionBtns, btn)
    end

    return actionBtns
end

local function SetButtonsLocked()
    local btnTable = GetButtons()
    local prevBtn = btnTable[1]

    -- Setup first button
    Ui:SetPointFinal(prevBtn, "BOTTOMLEFT", BaikMainMenuBar, 10, 5, true)

    for idx=2, NUM_ACTIONBAR_BUTTONS, 1 do
        curBtn = btnTable[idx]
        Ui:SetPointFinal(curBtn, "LEFT", prevBtn, "RIGHT", 6, 0, true)
        prevBtn = curBtn
    end
end

local function SetButtons()
    if CombatSafe:InLockdown() then
        return
    end

    SetButtonsLocked()
end

-- Ace3 Callbacks
function ActionBar:OnInitialize()
    Log:v("ActionBar_OnInitialize")
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(SetButtonsLocked)
    end
end

function ActionBar:OnEnable()
    Log:v("ActionBar_OnEnable")
end

function ActionBar:OnDisable()
    Log:v("ActionBar_OnDisable")
end
