local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option
local Ui = BaikCore.Ui
local VertexColor = BaikCore.VertexColor

local FrameStyle = BaikActionbar.FrameStyle

-- Add Module
local Art = BaikActionbar:NewModule("Art")

-- Constants
local DB_PREFIX = "action_art"
local OPTION_NAME = "action_art_style"
local ART_VIS_DB = "action_art_vis"

local FRAMES = {
    -- Gryphons
    BaikMainMenuBar.LeftEndCap,
    BaikMainMenuBar.RightEndCap,
    MainMenuBarArtFrame.LeftEndCap,
    MainMenuBarArtFrame.RightEndCap,

    -- Actionbar
    BaikMainMenuBar.Background,
    MainMenuBarArtFrameBackground.BackgroundSmall,
    MainMenuBarArtFrameBackground.BackgroundLarge,

    -- Statusbar
    StatusTrackingBarManager.SingleBarSmall,
    StatusTrackingBarManager.SingleBarSmallUpper,
    StatusTrackingBarManager.SingleBarLarge,
    StatusTrackingBarManager.SingleBarLargertUpper,

    -- Petbar
    SlidingActionBarTexture0,
    SlidingActionBarTexture1,

    -- Stance
    StanceBarLeft,
    StanceBarMiddle,
    StanceBarRight,
    StanceButton1NormalTexture2,
    StanceButton2NormalTexture2,
    StanceButton3NormalTexture2,
    StanceButton4NormalTexture2,
    StanceButton5NormalTexture2,
    StanceButton6NormalTexture2,
    StanceButton7NormalTexture2,
    StanceButton8NormalTexture2,
    StanceButton9NormalTexture2,
    StanceButton10NormalTexture2,
}

local GRYPHON = {
    MainMenuBarArtFrame.LeftEndCap,
    MainMenuBarArtFrame.RightEndCap,
}

local COMPACT_GRYPHON = {
    BaikMainMenuBar.LeftEndCap,
    BaikMainMenuBar.RightEndCap,
}

-- Private Methos
local function HideGryphonLocked()
    local gryphons = FrameStyle:IsCompact() and COMPACT_GRYPHON or GRYPHON
    local hide = Database:Get(ART_VIS_DB)
    for _, frame in pairs(gryphons) do
        if hide then
            Ui:HideFinal(frame)
        else
            Ui:UndoHideFinal(frame)
        end
    end
end

local function ColorFrames()
    VertexColor:ColorFrameAll(FRAMES, DB_PREFIX)
end

local function GetOpts()
    local opt = {
        name = "Art",
        type = "group",
        args = {
            vis = {
                name = "Hide Gryphons",
                type = "toggle",
                get = function(...)
                    return Database:Get(ART_VIS_DB)
                end,
                set = function(self, value, ...)
                    Database:Set(ART_VIS_DB, value)
                    if not CombatSafe:InLockdown() then
                        HideGryphonLocked()
                    end
                end
            }
        }
    }
    VertexColor:InsertOpts(opt.args, DB_PREFIX, ColorFrames)

    return opt
end

-- Ace3 Callbacks
function Art:OnInitialize()
    Log:v("Art_OnInitialize")
    VertexColor:InitDb(DB_PREFIX)
    Database:InitIfEmpty(ART_VIS_DB, false)
    BaikActionbar:AddOption(OPTION_NAME, GetOpts())
    CombatSafe:Setup(HideGryphonLocked)
end

function Art:OnEnable()
    Log:v("Art_OnEnable")
    ColorFrames()
end

function Art:OnDisable()
    Log:v("Art_OnDisable")
end
