local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option
local Visibility = BaikCore.Visibility
local Zoom = BaikCore.Zoom

local Border = BaikActionbar.Border

-- Add Module (Add prefix to avoid collision)
local BaikBag = BaikActionbar:NewModule("Bag")

-- Constants
local OPTION_NAME = "bag_style"
local DB_PREFIX = "action_bag"

local ICONS = {
    CharacterBag0SlotIconTexture,
    CharacterBag1SlotIconTexture,
    CharacterBag2SlotIconTexture,
    CharacterBag3SlotIconTexture,
    MainMenuBarBackpackButtonIconTexture,
}

local BORDERS = {
    CharacterBag0SlotNormalTexture,
    CharacterBag1SlotNormalTexture,
    CharacterBag2SlotNormalTexture,
    CharacterBag3SlotNormalTexture,
    CharacterBag0Slot.IconBorder,
    CharacterBag1Slot.IconBorder,
    CharacterBag2Slot.IconBorder,
    CharacterBag3Slot.IconBorder,
    MainMenuBarBackpackButtonNormalTexture,
}

local VISIBILITY = {
    CharacterBag0Slot,
    CharacterBag1Slot,
    CharacterBag2Slot,
    CharacterBag3Slot,
    MainMenuBarBackpackButton,
}

-- Private Variables
local BaikBagFrame = CreateFrame("Frame", "BaikBagFrame", UIParent)

-- Private Methods
local function ReParent()
    for _, frame in pairs(VISIBILITY) do
        frame:SetParent(BaikBagFrame)
    end
end
local function SetZoomed()
    Zoom:SetZoomTextures(DB_PREFIX, ICONS)
end

local function SetBorder()
    Border:RemoveFrames(BORDERS, DB_PREFIX)
end

local function SetVisibility()
    Visibility:SetupFrameAll(DB_PREFIX, VISIBILITY, BaikBagFrame)
end

local function GetOpts()
    return {
        name = "Bag",
        type = "group",
        args = {
            zoom = Zoom:GetOpts(DB_PREFIX, SetZoomed),
            border = Border:GetOpts(DB_PREFIX, SetBorder),
            vis = Visibility:GetOpts(DB_PREFIX, SetVisibility)
        }
    }
end

-- Ace3 Callbacks
function BaikBag:OnInitialize()
    Log:v("BaikBag_OnInitialize")
    Border:InitDb(DB_PREFIX)
    Visibility:InitDb(DB_PREFIX)
    Zoom:InitDb(DB_PREFIX)
    Option:AddOption(BaikActionbar.OPTION_NAME, OPTION_NAME, GetOpts())
    CombatSafe:Setup(ReParent)
end

function BaikBag:OnEnable()
    Log:v("BaikBag_OnEnable")
    MicroButtonAndBagsBar.MicroBagBar:SetAlpha(0)
end

function BaikBag:OnDisable()
    Log:v("BaikBag_OnDisable")
    self:UnhookAll()
end
