local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Ui = BaikCore.Ui

local FrameStyle = BaikActionbar.FrameStyle

-- New Module
local BottomRight = BaikActionbar:NewModule("BottomRight", "AceHook-3.0")

-- Constants
local ACTION_PREFIX = "MultiBarBottomRightButton%d"

-- Private Variables
local actionBtns = nil

-- Private Methods
local function GetButtons()
    if actionBtns then
        return actionBtns
    end

    actionBtns = {}
    for idx=1, NUM_ACTIONBAR_BUTTONS, 1 do
        local btnName = string.format(ACTION_PREFIX, idx)
        local btn = _G[btnName]
        Assert:NotNull(btn)
        table.insert(actionBtns, btn)
    end

    return actionBtns
end

local function SetButtonsLocked()
    local btnTable = GetButtons()
    local prevBtn = btnTable[1]

    -- Setup first button
    if MultiBarBottomLeft:IsShown() then
        Ui:SetPointFinal(prevBtn, "BOTTOMLEFT", BaikMainMenuBar, 10, 94, true)
    else
        Ui:SetPointFinal(prevBtn, "BOTTOMLEFT", BaikMainMenuBar, 10, 52, true)
    end

    for idx=2, NUM_ACTIONBAR_BUTTONS, 1 do
        curBtn = btnTable[idx]
        Ui:SetPointFinal(curBtn, "LEFT", prevBtn, "RIGHT", 6, 0, true)
        prevBtn = curBtn
    end
end
-- Private Methods
local function SetButtons()
    if CombatSafe:InLockdown() then
        return
    end

    SetButtonsLocked()
end

local function SetupHook()
    Hook:EnsureSecureHookScript(BottomRight, MultiBarBottomLeft, "OnShow", SetButtons)
    Hook:EnsureSecureHookScript(BottomRight, MultiBarBottomLeft, "OnHide", SetButtons)
end

-- Ace3 Callbacks
function BottomRight:OnInitialize()
    Log:v("BottomRight_OnInitialize")
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(SetButtonsLocked)
    end
end

function BottomRight:OnEnable()
    Log:v("BottomRight_OnEnable")
    if FrameStyle:IsCompact() then
        SetupHook()
    end
end

function BottomRight:OnDisable()
    Log:v("BottomRight_OnDisable")
end
