local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Option = BaikCore.Option
local Ui = BaikCore.Ui

local FrameStyle = BaikActionbar.FrameStyle

-- New Module
local BottomRightGrid = BaikActionbar:NewModule("BottomRightGrid", "AceHook-3.0", "AceEvent-3.0")

-- Constants
local BOTTOM_RIGHT_GRID = nil
local BOTTOM_RIGHT_EVENTS = nil

-- Private Methods
local function SetHotKeyParentLocked(btn, parent)
    Assert:Table(btn)
    btn.HotKey:SetParent(parent)
end

local function GetGridLocked()
    if BOTTOM_RIGHT_GRID then
        return BOTTOM_RIGHT_GRID
    end

    BOTTOM_RIGHT_GRID = {}

    local pattern = "MultiBarBottomRightButton%d"
    local texPattern = "%sFloatingBG"
    for idx = 1, 6, 1 do
        local btn = _G[string.format(pattern, idx)]
        local texName = string.format(texPattern, btn:GetName())

        -- Copied from FrameXML/MultiActionBars.xml
        local tex = MultiBarBottomRight:CreateTexture(texName, "BACKGROUND", nil, -1)
        tex:SetPoint("TOPLEFT", btn, -15, 15)
        tex:SetPoint("BOTTOMRIGHT", btn, 15, -15)
        tex:SetTexture("Interface\\Buttons\\UI-Quickslot")
        tex:SetAlpha(0.4)

        -- Ensure we reassign hotkey parent
        local function BtnParent()
            if CombatSafe:InLockdown() then
                return
            end
            SetHotKeyParentLocked(btn, btn)
        end
        Hook:EnsureSecureHookScript(BottomRightGrid, btn, "OnShow", BtnParent)

        local function GridParent()
            if CombatSafe:InLockdown() then
                return
            end

            if MultibarGrid_IsVisible() and not btn:IsShown() then
                SetHotKeyParentLocked(btn, MultiBarBottomRight)
            end
        end
        Hook:EnsureSecureHookScript(BottomRightGrid, btn, "OnHide", GridParent)

        BOTTOM_RIGHT_GRID[btn] = tex
    end

    return BOTTOM_RIGHT_GRID
end

local function ShowAllGrid()
    Log:d("BottomRightGrid_ShowAllGrid()")
    if CombatSafe:InLockdown() then
        return
    end

    local gridFrames = GetGridLocked()
    for btn, texture in pairs(gridFrames) do
        texture:Show()
        if not btn:IsShown() then
            SetHotKeyParentLocked(btn, MultiBarBottomRight)
        end
    end
end

local function HideAllGrid()
    Log:d("BottomRightGrid_HideAllGrid()")
    if CombatSafe:InLockdown() then
        return
    end

    local gridFrame = GetGridLocked()
    for btn, texture in pairs(gridFrame) do
        texture:Hide()
        SetHotKeyParentLocked(btn, btn)
    end
end

local function SafeHideAllGrid()
    Log:d("BottomRightGrid_SafeHideAllGrid()")
    -- Ignore hide
    if MultibarGrid_IsVisible() then
        return
    end

    HideAllGrid()
end

local function SetupHook()
    Hook:EnsureSecureHook(BottomRightGrid, nil, "MultiActionBar_HideAllGrids", SafeHideAllGrid)
    Hook:EnsureSecureHook(BottomRightGrid, nil, "MultiActionBar_ShowAllGrids", ShowAllGrid)
end

-- Lazy load
local function GetEvents()
    if BOTTOM_RIGHT_EVENTS then
        return BOTTOM_RIGHT_EVENTS
    end

    BOTTOM_RIGHT_EVENTS = {
        ["ACTIONBAR_SHOWGRID"] = ShowAllGrid,
        ["ACTIONBAR_HIDEGRID"] = SafeHideAllGrid,
    }

    return BOTTOM_RIGHT_EVENTS
end

local function RegisterEvents()
    local events = GetEvents()
    for event, handler in pairs(events) do
        BottomRightGrid:RegisterEvent(event, handler)
    end
end

local function UnregisterEvents()
    local events = GetEvents()
    for event, _ in pairs(events) do
        BottomRightGrid:UnregisterEvent(event)
    end
end

-- Ace3 Callbacks
function BottomRightGrid:OnInitialize()
    Log:v("BottomRight_OnInitialize")
end

function BottomRightGrid:OnEnable()
    Log:v("BottomRightGrid_OnEnable")
    if FrameStyle:IsCompact() then
        SetupHook()
        RegisterEvents()
    end
end

function BottomRightGrid:OnDisable()
    Log:v("BottomRightGrid_OnDisable")
end

