local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option
local Reload = BaikCore.Reload

-- Add Module
local FrameStyle = BaikActionbar:NewModule("FrameStyle")
BaikActionbar.FrameStyle = FrameStyle

-- Constants
local FRAME_STYLE_DB = "action_frame_style"

local COMPACT_KEY = 1
local BLIZZARD_KEY = 2

local FRAME_OPTS = {
    "Compact",
    "Blizzard"
}

-- Private Variables
local syncFunc = {}

-- Private Methods
local function SyncLocked()
    for _, func in pairs(syncFunc) do
        func()
    end
end

local function Sync()
    if CombatSafe:InLockdown() then
        return
    end

    SyncLocked()
end

local function FrameOpts()
    return {
        name = "Frame Style",
        type = "select",
        values = FRAME_OPTS,
        order = -1,
        get = function(...)
            return Database:Get(FRAME_STYLE_DB)
        end,
        set = function(self, key, value, ...)
            local curState = Database:Get(FRAME_STYLE_DB)

            if key ~= curState then
                Database:Set(FRAME_STYLE_DB, key)
                Reload:Enable()
            end
        end
    }
end

-- FrameStyle API
function FrameStyle:AddSync(func)
    table.insert(syncFunc, func)
end

function FrameStyle:IsCompact()
    return Database:Get(FRAME_STYLE_DB) == COMPACT_KEY
end

function FrameStyle:IsBlizzard()
    return Database:Get(FRAME_STYLE_DB) == BLIZZARD_KEY
end

-- Ace3 Callbacks
function FrameStyle:OnInitialize()
    Log:v("FrameStyle_OnInitialize")
    Database:InitIfEmpty(FRAME_STYLE_DB, COMPACT_KEY)
    Option:AddOption(BaikActionbar.OPTION_NAME, "frame_style", FrameOpts())
    CombatSafe:Setup(SyncLocked)
end

function FrameStyle:OnEnable()
    Log:v("FrameStyle_OnEnable")
end

function FrameStyle:OnDisable()
    Log:v("FrameStyle_OnDisable")
end
