local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option
local Reload = BaikCore.Reload
local Table = BaikCore.Table
local Visibility = BaikCore.Visibility
local Ui = BaikCore.Ui
local Zoom = BaikCore.Zoom

local Border = BaikActionbar.Border
local FrameStyle = BaikActionbar.FrameStyle

-- Add Module
local MenuBar = BaikActionbar:NewModule("MenuBar")
BaikActionbar.MenuBar = MenuBar

-- Constants
local OPTION_NAME = "menu_bar_style"
local DB_PREFIX = "action_menu_bar"

local BUTTONS = nil

local BUTTON_PREFIX = {
    "ActionButton%d",
    "MultiBarBottomLeftButton%d",
    "MultiBarBottomRightButton%d",
    "MultiBarLeftButton%d",
    "MultiBarRightButton%d",
}

local VISIBILITY = nil

local VISIBILITY_EXTRA = {
    MainMenuBarArtFrame,
    StatusTrackingBarManager
}

local COMPACT_HIDE = {
    MainMenuBarArtFrameBackground,
    MainMenuBarArtFrame.LeftEndCap,
    MainMenuBarArtFrame.RightEndCap,
    MainMenuBarArtFrame.PageNumber
}

-- Private Methods
local function GetButtons()
    -- Already generated buttons, exit early
    if BUTTONS then
        return BUTTONS
    end

    -- Build table
    local btnTable = {}
    for _, fmt in pairs(BUTTON_PREFIX) do
        for idx = 1, 12, 1 do
            local btnName = string.format(fmt, idx)
            local btn = _G[btnName]
            if not btn then
                Log:i("%s button is nil", btnName)
            else
                table.insert(btnTable, btn)
            end
        end
    end

    BUTTONS = btnTable
    return BUTTONS
end

local function GetVisibility()
    -- Already generated buttons, exit early
    if VISIBILITY then
        return VISIBILITY
    end

    local visTable = {}
    local btns = GetButtons()
    Table:Append(visTable, btns)
    Table:Append(visTable, VISIBILITY_EXTRA)
    VISIBILITY = visTable

    return visTable
end

local function HandleBlizzFrames()
    Ui:HideFinalAll(COMPACT_HIDE)
    StatusTrackingBarManager:UnregisterAllEvents()
    StatusTrackingBarManager:Hide()
    Ui:SetPointFinal(ActionBarUpButton, "RIGHT", BaikMainMenuBar, "RIGHT", -19, 7, true)
end

local function SetZoomed()
    local btns = GetButtons()
    Zoom:SetZoomButtons(DB_PREFIX, btns)
end

local function SetBorder()
    local btns = GetButtons()

    Border:RemoveButtonAll(btns, DB_PREFIX)
end

local function SetVisibility()
    Visibility:SetupFrameAll(DB_PREFIX, GetVisibility(), MainMenuBar)
end

local function GetOpts()
    return {
        name = "MainMenuBar",
        type = "group",
        args = {
            zoom = Zoom:GetOpts(DB_PREFIX, SetZoomed),
            border = Border:GetOpts(DB_PREFIX, SetBorder),
            vis = Visibility:GetOpts(DB_PREFIX, SetVisibility)
        }
    }
end

function MenuBar:IsCompact()
    return IsCompactEnabled()
end

-- Ace3 Callbacks
function MenuBar:OnInitialize()
    Log:v("MenuBar_OnInitialize")
    Border:InitDb(DB_PREFIX)
    Visibility:InitDb(DB_PREFIX)
    Zoom:InitDb(DB_PREFIX)
    Option:AddOption(BaikActionbar.OPTION_NAME, OPTION_NAME, GetOpts())
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(HandleBlizzFrames)
    end
end

function MenuBar:OnEnable()
    Log:v("MenuBar_OnEnable")
end

function MenuBar:OnDisable()
    Log:v("MenuBar_OnDisable")
end
