local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Hook = BaikCore.Hook
local Option = BaikCore.Option
local Ui = BaikCore.Ui
local Visibility = BaikCore.Visibility
local Zoom = BaikCore.Zoom

local Border = BaikActionbar.Border
local FrameStyle = BaikActionbar.FrameStyle

-- Add Module
local BaikPet = BaikActionbar:NewModule("Pet", "AceHook-3.0")

-- Constants
local OPTION_NAME = "pet_style"
local DB_PREFIX = "action_pet"

local ARTS = {
    SlidingActionBarTexture0,
    SlidingActionBarTexture1
}

local ICONS = {
    PetActionButton1Icon,
    PetActionButton2Icon,
    PetActionButton3Icon,
    PetActionButton4Icon,
    PetActionButton5Icon,
    PetActionButton6Icon,
    PetActionButton7Icon,
    PetActionButton8Icon,
    PetActionButton9Icon,
    PetActionButton10Icon,
    PetActionButton1:GetCheckedTexture(),
    PetActionButton2:GetCheckedTexture(),
    PetActionButton3:GetCheckedTexture(),
    PetActionButton4:GetCheckedTexture(),
    PetActionButton5:GetCheckedTexture(),
    PetActionButton6:GetCheckedTexture(),
    PetActionButton7:GetCheckedTexture(),
    PetActionButton8:GetCheckedTexture(),
    PetActionButton9:GetCheckedTexture(),
    PetActionButton10:GetCheckedTexture(),
}

local BORDERS = {
    PetActionButton1NormalTexture2,
    PetActionButton2NormalTexture2,
    PetActionButton3NormalTexture2,
    PetActionButton4NormalTexture2,
    PetActionButton5NormalTexture2,
    PetActionButton6NormalTexture2,
    PetActionButton7NormalTexture2,
    PetActionButton8NormalTexture2,
    PetActionButton9NormalTexture2,
    PetActionButton10NormalTexture2,
}

local VISIBILITY = {
    PetActionButton1,
    PetActionButton2,
    PetActionButton3,
    PetActionButton4,
    PetActionButton5,
    PetActionButton6,
    PetActionButton7,
    PetActionButton8,
    PetActionButton9,
    PetActionButton10,
    PetActionBarFrame,
}

-- Private Methods
local function GetYOffset(leftShown, rightShown)

    -- Constants
    local DEFAULT = 92 -- Blizzard incorrectly uses yPos = 89 :P
    local BAR_GAP = 4
    local EXTRA_SIZE = 36

    -- Local variables
    local gapCount = 0
    local extraCount = 0

    if leftShown or rightShown then
        -- Need a little gap if it's off the mainbar
        gapCount = gapCount + 1

        if leftShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end

        if rightShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end
    end

    return DEFAULT + (gapCount * BAR_GAP) + (extraCount * EXTRA_SIZE)
end

local function SetupLocked()
    Log:d("BaikActionbar_SetupLocked")

    local leftShown = MultiBarBottomLeft:IsShown()
    local rightShown = MultiBarBottomRight:IsShown()

    Border:RemoveAlphas(BORDERS, DB_PREFIX, rightShown or leftShown)
    Border:RemoveFrames(ARTS, DB_PREFIX, rightShown or leftShown)

    local yPos = GetYOffset(leftShown, rightShown)
    Ui:SetPointFinal(PetActionBarFrame, "TOPLEFT", BaikMainMenuBar, "BOTTOMLEFT", 40, yPos, true)
end

local function Setup()
    Log:v("BaikPet_SetupPetFrame")
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

local function SetupHook()
    Hook:EnsureSecureHookScript(BaikPet, PetActionBarFrame, "OnShow", Setup)
    Hook:EnsureSecureHookScript(BaikPet, MultiBarBottomLeft, "OnShow", Setup)
    Hook:EnsureSecureHookScript(BaikPet, MultiBarBottomLeft, "OnHide", Setup)
    Hook:EnsureSecureHookScript(BaikPet, MultiBarBottomRight, "OnShow", Setup)
    Hook:EnsureSecureHookScript(BaikPet, MultiBarBottomRight, "OnHide", Setup)
end

local function SetZoomed()
    Zoom:SetZoomTextures(DB_PREFIX, ICONS)
end

local function SetBorder()
    Border:RemoveAlphas(BORDERS, DB_PREFIX)
    Border:RemoveFrames(ARTS, DB_PREFIX)
end

local function SetVisibility()
    Visibility:SetupFrameAll(DB_PREFIX, VISIBILITY, PetActionBarFrame)
end

local function GetOpts()
    return {
        name = "Pet",
        type = "group",
        args = {
            zoom = Zoom:GetOpts(DB_PREFIX, SetZoomed),
            border = Border:GetOpts(DB_PREFIX, SetBorder),
            vis = Visibility:GetOpts(DB_PREFIX, SetVisibility)
        }
    }
end

-- Ace3 Callbacks
function BaikPet:OnInitialize()
    Log:v("BaikPet_OnInitialize")
    Border:InitDb(DB_PREFIX)
    Visibility:InitDb(DB_PREFIX)
    Zoom:InitDb(DB_PREFIX)
    Option:AddOption(BaikActionbar.OPTION_NAME, OPTION_NAME, GetOpts())
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(SetupLocked)
    end
end

function BaikPet:OnEnable()
    Log:v("BaikPet_OnEnable")
    if FrameStyle:IsCompact() then
        SetupHook()
    end
end

function BaikPet:OnDisable()
    Log:v("BaikPet_OnDisable")
end
