local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Hook = BaikCore.Hook
local Option = BaikCore.Option
local Ui = BaikCore.Ui
local Visibility = BaikCore.Visibility
local Zoom = BaikCore.Zoom

local Border = BaikActionbar.Border
local FrameStyle = BaikActionbar.FrameStyle

-- Add Module (Add prefix to avoid collision)
local Possess = BaikActionbar:NewModule("Possess", "AceHook-3.0")

-- Constants
local OPTION_NAME = "possess_style"
local DB_PREFIX = "action_possess"

local ART = {
    PossessBackground1,
    PossessBackground2,
    PossessButton1NormalTexture,
    PossessButton2NormalTexture,
}

local BUTTONS = {
    PossessButton1,
    PossessButton2,
}

-- Private Methods
local function GetYOffset(leftShown, rightShown)
    -- Constants
    local DEFAULT = -1 -- Blizzard incorrectly uses yPos = 0 :P
    local BAR_GAP = 4
    local EXTRA_SIZE = 36

    -- Local variables
    local gapCount = 0
    local extraCount = 0

    if leftShown or rightShown then
        -- Need a little gap if it's off the mainbar
        gapCount = gapCount + 1

        if leftShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end

        if rightShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end
    end

    return DEFAULT + (gapCount * BAR_GAP) + (extraCount * EXTRA_SIZE)
end

local function SetupLocked()
    Log:v("Possess_SetupLocked")

    local leftShown = MultiBarBottomLeft:IsShown()
    local rightShown = MultiBarBottomRight:IsShown()

    Border:RemoveAlphas(ART, DB_PREFIX, rightShown or leftShown)

    local yPos = GetYOffset(leftShown, rightShown)
    Ui:SetPointFinal(PossessBarFrame, "BOTTOMLEFT", BaikMainMenuBar, "TOPLEFT", 30, yPos, true)
end

local function Setup()
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

local function SetupHook()
    Hook:EnsureSecureHookScript(Possess, PossessBarFrame, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Possess, MultiBarBottomLeft, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Possess, MultiBarBottomLeft, "OnHide", Setup)
    Hook:EnsureSecureHookScript(Possess, MultiBarBottomRight, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Possess, MultiBarBottomRight, "OnHide", Setup)
end

local function SetZoomed()
    Zoom:SetZoomButtons(DB_PREFIX, BUTTONS)
end

local function SetBorder()
    Border:RemoveFrames(ART, DB_PREFIX)
end

local function SetVisibility()
    Visibility:SetupFrameAll(DB_PREFIX, BUTTONS, PossessBarFrame)
end

local function GetOpts()
    return {
        name = "Possess",
        type = "group",
        args = {
            zoom = Zoom:GetOpts(DB_PREFIX, SetZoomed),
            border = Border:GetOpts(DB_PREFIX, SetBorder),
            vis = Visibility:GetOpts(DB_PREFIX, SetVisibility)
        }
    }
end

-- Ace3 Callbacks
function Possess:OnInitialize()
    Log:v("Possess_OnInitialize")
    Border:InitDb(DB_PREFIX)
    Visibility:InitDb(DB_PREFIX)
    Zoom:InitDb(DB_PREFIX)
    Option:AddOption(BaikActionbar.OPTION_NAME, OPTION_NAME, GetOpts())
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(SetupLocked)
    end
end

function Possess:OnEnable()
    Log:v("Possess_OnEnable")
    if FrameStyle:IsCompact() then
        SetupHook()
    end
end

function Possess:OnDisable()
    Log:v("Possess_OnDisable")
end
