local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Hook = BaikCore.Hook
local Option = BaikCore.Option
local Ui = BaikCore.Ui
local Visibility = BaikCore.Visibility
local Zoom = BaikCore.Zoom

local Border = BaikActionbar.Border
local FrameStyle = BaikActionbar.FrameStyle

-- Add Module
local Stance = BaikActionbar:NewModule("Stance", "AceHook-3.0")

-- Constants
local OPTION_NAME = "stance_style"
local DB_PREFIX = "action_stance"

-- Constants
local ART = {
    StanceBarLeft,
    StanceBarMiddle,
    StanceBarRight,
    StanceButton1NormalTexture2,
    StanceButton2NormalTexture2,
    StanceButton3NormalTexture2,
    StanceButton4NormalTexture2,
    StanceButton5NormalTexture2,
    StanceButton6NormalTexture2,
    StanceButton7NormalTexture2,
    StanceButton8NormalTexture2,
    StanceButton9NormalTexture2,
    StanceButton10NormalTexture2,
}

local BUTTONS = {
    StanceButton1,
    StanceButton2,
    StanceButton3,
    StanceButton4,
    StanceButton5,
    StanceButton6,
    StanceButton7,
    StanceButton8,
    StanceButton9,
    StanceButton10,
}

-- Public Methods
local BaikStanceFrame = CreateFrame("FRAME", "BaikStanceFrame", BaikMainMenuBar)

-- Private Methods
local function ReParent()
    for _, btn in pairs(BUTTONS) do
        btn:SetParent(BaikStanceFrame)
    end
end

local function GetYOffset(leftShown, rightShown)
    -- Constants
    local DEFAULT = -1 -- Blizzard incorrectly uses yPos = 0 :P
    local BAR_GAP = 4
    local EXTRA_SIZE = 36

    -- Local variables
    local gapCount = 0
    local extraCount = 0

    if leftShown or rightShown then
        -- Need a little gap if it's off the mainbar
        gapCount = gapCount + 1

        if leftShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end

        if rightShown then
            gapCount = gapCount + 1
            extraCount = extraCount + 1
        end
    end

    return DEFAULT + (gapCount * BAR_GAP) + (extraCount * EXTRA_SIZE)
end

local function SetupLocked()
    local leftShown = MultiBarBottomLeft:IsShown()
    local rightShown = MultiBarBottomRight:IsShown()

    Border:RemoveAlphas(ART, DB_PREFIX, rightShown or leftShown)

    local yPos = GetYOffset(leftShown, rightShown)
    Ui:SetPointFinal(StanceBarFrame, "BOTTOMLEFT", BaikMainMenuBar, "TOPLEFT", 30, yPos, true)
end

local function Setup()
    Log:v("Stance_Setup")
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

local function SetupHook()
    Hook:EnsureSecureHookScript(Stance, StanceBarFrame, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Stance, MultiBarBottomLeft, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Stance, MultiBarBottomLeft, "OnHide", Setup)
    Hook:EnsureSecureHookScript(Stance, MultiBarBottomRight, "OnShow", Setup)
    Hook:EnsureSecureHookScript(Stance, MultiBarBottomRight, "OnHide", Setup)
end

local function SetZoomed()
    Zoom:SetZoomButtons(DB_PREFIX, BUTTONS)
end

local function SetBorder()
    Border:RemoveAlphas(ART, DB_PREFIX)
end

local function SetVisibility()
    Visibility:SetupFrameAll(DB_PREFIX, BUTTONS, BaikStanceFrame)
end

local function GetOpts()
    return {
        name = "Stance",
        type = "group",
        args = {
            zoom = Zoom:GetOpts(DB_PREFIX, SetZoomed),
            border = Border:GetOpts(DB_PREFIX, SetBorder),
            vis = Visibility:GetOpts(DB_PREFIX, SetVisibility)
        }
    }
end

-- Ace3 Callbacks
function Stance:OnInitialize()
    Log:v("Stance_OnInitialize")
    Border:InitDb(DB_PREFIX)
    Visibility:InitDb(DB_PREFIX)
    Zoom:InitDb(DB_PREFIX)
    Option:AddOption(BaikActionbar.OPTION_NAME, OPTION_NAME, GetOpts())
    if FrameStyle:IsCompact() then
        CombatSafe:Setup(SetupLocked)
        CombatSafe:Setup(ReParent)
    end
end

function Stance:OnEnable()
    Log:v("Stance_OnEnable")
    if FrameStyle:IsCompact() then
        SetupHook()
    end
end

function Stance:OnDisable()
    Log:v("Stance_OnDisable")
end
