
BaikAzeriteBarMixin = CreateFromMixins(BaikStatusBarMixin)

function BaikAzeriteBarMixin:RegisterParent(parent)
    -- Do nothing
end

function BaikAzeriteBarMixin:ShouldBeVisible()
    return C_AzeriteItem.HasActiveAzeriteItem() and self:GetAzeriteOption()
end

function BaikAzeriteBarMixin:UpdateText()
    if ( self.OverlayFrame.Text:IsShown() ) then
        local xp = self.StatusBar:GetAnimatedValue()
        local _, totalLevelXP = self.StatusBar:GetMinMaxValues()
        self.OverlayFrame.Text:SetFormattedText("Azerite Power %d / %d", xp,  totalLevelXP, true)
    end
end

function BaikAzeriteBarMixin:UpdateBar()
    local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem()

    if (not azeriteItemLocation) then
        return
    end

    local azeriteItem = Item:CreateFromItemLocation(azeriteItemLocation)

    self.xp, self.totalLevelXP = C_AzeriteItem.GetAzeriteItemXPInfo(azeriteItemLocation)
    self.currentLevel = C_AzeriteItem.GetPowerLevel(azeriteItemLocation)
    self.xpToNextLevel = self.totalLevelXP - self.xp

    self:SetBarValues(self.xp, 0, self.totalLevelXP)
    self:Show()
end

function BaikAzeriteBarMixin:Update()
    if not self:ShouldBeVisible() then
        self:Hide()
        return
    end
    self:UpdateBar()
    self:UpdateText()
end

local EVENT_MAP = {
    ["PLAYER_ENTERING_WORLD"] = BaikAzeriteBarMixin.Update,
    ["PLAYER_XP_UPDATE"] = BaikAzeriteBarMixin.Update,
}

function BaikAzeriteBarMixin:AnimatedValueChangedCallback()
    self:UpdateText()
end

function BaikAzeriteBarMixin:OnEvent(event, ...)
    local func = EVENT_MAP[event]

    if not func then
        print("BaikAzeriteBarMixin: Unhandled event", event)
    end
    func(self)
end

function BaikAzeriteBarMixin:OnShow()
    for event, _ in pairs(EVENT_MAP) do
        self:RegisterEvent(event)
    end
    self:Update()
end

function BaikAzeriteBarMixin:OnHide()
    for event, _ in pairs(EVENT_MAP) do
        self:UnregisterEvent(event)
    end
end

function BaikAzeriteBarMixin:OnLoad()
    self:SetBarColor(ARTIFACT_BAR_COLOR:GetRGB())
    self.StatusBar:SetOnAnimatedValueChangedCallback(function() self:AnimatedValueChangedCallback() end)
    self.priority = 3
    self:GetParent():AddBar(self)
end
