BaikExpBarMixin = CreateFromMixins(BaikStatusBarMixin)

function BaikExpBarMixin:RegisterParent(parent)
    parent:RegisterEvent("PLAYER_LEVEL_UP")
    parent:RegisterEvent("UPDATE_EXPANSION_LEVEL")
end

function BaikExpBarMixin:ShouldBeVisible()
    local level = UnitLevel("player")
    return level < MAX_PLAYER_LEVEL and not IsXPUserDisabled() and self:GetXpOption()
end

function BaikExpBarMixin:UpdateText()
    local currXP = self.currXP
    local maxBar = self.maxBar
    if (GameLimitedMode_IsActive()) then
        local rLevel = GetRestrictedAccountData()
        if (UnitLevel("player") >= rLevel) then
            currXP = UnitTrialXP("player")
        end
    end
    self:SetBarText(XP_STATUS_BAR_TEXT:format(currXP, maxBar))
end

function BaikExpBarMixin:UpdateBar()
    local currXP = UnitXP("player")
    local nextXP = UnitXPMax("player")
    local level = UnitLevel("player")

    local minBar, maxBar = 0, nextXP

    local isCapped = false
    if (GameLimitedMode_IsActive()) then
        local rLevel = GetRestrictedAccountData()
        if UnitLevel("player") >= rLevel then
            isCapped = true
            self:SetBarValues(1, 0, 1, level)
            self.StatusBar:ProcessChangesInstantly()
            self:SetBarColor(0.58, 0.0, 0.55, 1.0)
        end
    end
    if (not isCapped) then
        self:SetBarValues(currXP, minBar, maxBar, level)
    end

    self.currXP = currXP
    self.maxBar = maxBar
end

function BaikExpBarMixin:Update()
    if not self:ShouldBeVisible() then
        return
    end
    self:UpdateBar()
    self:UpdateText()
end

local EVENT_MAP = {
    ["PLAYER_ENTERING_WORLD"] = BaikExpBarMixin.Update,
    ["PLAYER_XP_UPDATE"] = BaikExpBarMixin.Update,
}

function BaikExpBarMixin:OnEvent(event, ...)
    local func = EVENT_MAP[event]

    if not func then
        print("BaikExpBarMixin: Unhandled event", event)
    end
    func(self)
end

function BaikExpBarMixin:OnShow()
    self:Update()
end

function BaikExpBarMixin:OnLoad()
    self.priority = 0
    self.maxLevel = GetMaxPlayerLevel()
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_XP_UPDATE")
    self:GetParent():AddBar(self)
end
