BaikTrackingBarMixin = {}

function BaikTrackingBarMixin:GetVisibleBars()
    local visBars = {}
    for _, bar in pairs(self.bars) do
        if bar:ShouldBeVisible() then
            table.insert(visBars, bar)
        end
    end
    table.sort(visBars, function(left, right) return left:GetPriority() < right:GetPriority() end)
    return visBars
end

function BaikTrackingBarMixin:SetupBars()
    local visBars = self:GetVisibleBars()
    local nextBar = visBars[1]

    if self.curBar == nextBar then
        return
    end

    if self.curBar then
        self.curBar:Hide()
    end

    if nextBar then
        nextBar:Show()
    end

    if not nextBar and self.curBar then
        self:ShiftMainMenuBar(false)
    elseif nextBar and not self.curBar then
        self:ShiftMainMenuBar(true)
    end

    self.curBar = nextBar
end

function BaikTrackingBarMixin:AddBar(bar)
    table.insert(self.bars, bar)
    bar:RegisterParent(self)
end

function BaikTrackingBarMixin:ShiftMainMenuBar(show)
    local height = (show and 1 or -1) * self.height

    -- Shift MainBar
    local point, relativeTo, relativePoint, xOff, yOff = BaikMainMenuBar:GetPoint()
    BaikMainMenuBar:SetPoint(point, relativeTo, relativePoint, xOff, yOff + height)

    -- Shift Endcaps
    point, relativeTo, relativePoint, xOff, yOff = BaikMainMenuBar.LeftEndCap:GetPoint()
    BaikMainMenuBar.LeftEndCap:SetPoint(point, relativeTo, relativePoint, xOff, yOff - height)

    point, relativeTo, relativePoint, xOff, yOff = BaikMainMenuBar.RightEndCap:GetPoint()
    BaikMainMenuBar.RightEndCap:SetPoint(point, relativeTo, relativePoint, xOff, yOff - height)
end

function BaikTrackingBarMixin:OnEvent(event, ...)
    self:SetupBars()
end

function BaikTrackingBarMixin:OnHide()
    --self:ShiftMainMenuBar(false)
end

function BaikTrackingBarMixin:OnShow()
    --self:ShiftMainMenuBar(true)
end

function BaikTrackingBarMixin:OnLoad()
    self.bars = {}
    self.curBar = nil
    self.height = self:GetHeight()
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
end
